@@ -10,19 +10,20 @@ import os.path as osp
 from os.path import join as pjoin
 import sys
 
-from jupyter_core.application import JupyterApp, base_aliases
+from jupyter_core.application import JupyterApp, base_aliases, base_flags
 from jupyterlab_server import slugify, WORKSPACE_EXTENSION
 from notebook.notebookapp import NotebookApp, aliases, flags
 from notebook.utils import url_path_join as ujoin
-from traitlets import Bool, Unicode
+from traitlets import Bool, Instance, Unicode
 
 from ._version import __version__
 from .debuglog import DebugLogFileMixin
 from .extension import load_config, load_jupyter_server_extension
 from .commands import (
     build, clean, get_app_dir, get_app_version, get_user_settings_dir,
-    get_workspaces_dir
+    get_workspaces_dir, AppOptions,
 )
+from .coreconfig import CoreConfig
 
 
 build_aliases = dict(base_aliases)
@@ -30,6 +31,7 @@ build_aliases['app-dir'] = 'LabBuildApp.app_dir'
 build_aliases['name'] = 'LabBuildApp.name'
 build_aliases['version'] = 'LabBuildApp.version'
 build_aliases['dev-build'] = 'LabBuildApp.dev_build'
+build_aliases['minimize'] = 'LabBuildApp.minimize'
 build_aliases['debug-log-path'] = 'DebugLogFileMixin.debug_log_path'
 
 build_flags = dict(flags)
@@ -39,6 +41,42 @@ app_version = get_app_version()
 if version != app_version:
     version = '%s (dev), %s (app)' % (__version__, app_version)
 
+buildFailureMsg = """Build failed.
+Troubleshooting: If the build failed due to an out-of-memory error, you
+may be able to fix it by disabling the `dev_build` and/or `minimize` options.
+
+If you are building via the `jupyter lab build` command, you can disable
+these options like so:
+
+jupyter lab build --dev-build=False --minimize=False
+
+You can also disable these options for all JupyterLab builds by adding these
+lines to a Jupyter config file named `jupyter_config.py`:
+
+c.LabBuildApp.minimize = False
+c.LabBuildApp.dev_build = False
+
+If you don't already have a `jupyter_config.py` file, you can create one by
+adding a blank file of that name to any of the Jupyter config directories.
+The config directories can be listed by running:
+
+jupyter --paths
+
+Explanation:
+
+- `dev-build`: This option controls whether a `dev` or a more streamlined
+`production` build is used. This option will default to `False` (ie the
+`production` build) for most users. However, if you have any labextensions
+installed from local files, this option will instead default to `True`.
+Explicitly setting `dev-build` to `False` will ensure that the `production`
+build is used in all circumstances.
+
+- `minimize`: This option controls whether your JS bundle is minified
+during the Webpack build, which helps to improve JupyterLab's overall
+performance. However, the minifier plugin used by Webpack is very memory
+intensive, so turning it off may help the build finish successfully in
+low-memory environments.
+"""
 
 class LabBuildApp(JupyterApp, DebugLogFileMixin):
     version = version
@@ -52,6 +90,9 @@ class LabBuildApp(JupyterApp, DebugLogFileMixin):
     aliases = build_aliases
     flags = build_flags
 
+    # Not configurable!
+    core_config = Instance(CoreConfig, allow_none=True)
+
     app_dir = Unicode('', config=True,
         help="The app directory to build in")
 
@@ -61,43 +102,102 @@ class LabBuildApp(JupyterApp, DebugLogFileMixin):
     version = Unicode('', config=True,
         help="The version of the built application")
 
-    dev_build = Bool(True, config=True,
-        help="Whether to build in dev mode (defaults to dev mode)")
+    dev_build = Bool(None, allow_none=True, config=True,
+        help="Whether to build in dev mode. Defaults to True (dev mode) if there are any locally linked extensions, else defaults to False (prod mode).")
+
+    minimize = Bool(True, config=True,
+        help="Whether to use a minifier during the Webpack build (defaults to True). Only affects production builds.")
 
     pre_clean = Bool(False, config=True,
         help="Whether to clean before building (defaults to False)")
 
     def start(self):
-        command = 'build:prod' if not self.dev_build else 'build'
+        parts = ['build']
+        parts.append('none' if self.dev_build is None else
+                     'dev' if self.dev_build else
+                     'prod')
+        if self.minimize:
+            parts.append('minimize')
+        command = ':'.join(parts)
+
         app_dir = self.app_dir or get_app_dir()
+        app_options = AppOptions(
+            app_dir=app_dir, logger=self.log, core_config=self.core_config
+        )
         self.log.info('JupyterLab %s', version)
         with self.debug_logging():
             if self.pre_clean:
                 self.log.info('Cleaning %s' % app_dir)
-                clean(self.app_dir)
+                clean(app_options=app_options)
             self.log.info('Building in %s', app_dir)
-            build(app_dir=app_dir, name=self.name, version=self.version,
-                command=command, logger=self.log)
+            try:
+                build(name=self.name, version=self.version,
+                  command=command, app_options=app_options)
+            except Exception as e:
+                print(buildFailureMsg)
+                raise e
 
 
 clean_aliases = dict(base_aliases)
 clean_aliases['app-dir'] = 'LabCleanApp.app_dir'
 
+ext_warn_msg = "WARNING: this will delete all of your extensions, which will need to be reinstalled"
+
+clean_flags = dict(base_flags)
+clean_flags['extensions'] = ({'LabCleanApp': {'extensions': True}},
+    'Also delete <app-dir>/extensions.\n%s' % ext_warn_msg)
+clean_flags['settings'] = ({'LabCleanApp': {'settings': True}}, 'Also delete <app-dir>/settings')
+clean_flags['static'] = ({'LabCleanApp': {'static': True}}, 'Also delete <app-dir>/static')
+clean_flags['all'] = ({'LabCleanApp': {'all': True}},
+    'Delete the entire contents of the app directory.\n%s' % ext_warn_msg)
+
+
+class LabCleanAppOptions(AppOptions):
+    extensions = Bool(False)
+    settings = Bool(False)
+    staging = Bool(True)
+    static = Bool(False)
+    all = Bool(False)
+
 
 class LabCleanApp(JupyterApp):
     version = version
     description = """
     Clean the JupyterLab application
 
-    This will clean the app directory by removing the `staging` and `static`
-    directories.
+    This will clean the app directory by removing the `staging` directories.
+    Optionally, the `extensions`, `settings`, and/or `static` directories,
+    or the entire contents of the app directory, can also be removed.
     """
     aliases = clean_aliases
+    flags = clean_flags
+
+    # Not configurable!
+    core_config = Instance(CoreConfig, allow_none=True)
 
     app_dir = Unicode('', config=True, help='The app directory to clean')
 
+    extensions = Bool(False, config=True,
+        help="Also delete <app-dir>/extensions.\n%s" % ext_warn_msg)
+
+    settings = Bool(False, config=True, help="Also delete <app-dir>/settings")
+
+    static = Bool(False, config=True, help="Also delete <app-dir>/static")
+
+    all = Bool(False, config=True,
+        help="Delete the entire contents of the app directory.\n%s" % ext_warn_msg)
+
     def start(self):
-        clean(self.app_dir, logger=self.log)
+        app_options = LabCleanAppOptions(
+            logger=self.log,
+            core_config=self.core_config,
+            app_dir=self.app_dir,
+            extensions=self.extensions,
+            settings=self.settings,
+            static=self.static,
+            all=self.all
+        )
+        clean(app_options=app_options)
 
 
 class LabPathApp(JupyterApp):
@@ -114,7 +214,6 @@ class LabPathApp(JupyterApp):
         environment variable or it will fall back to
         '/lab/workspaces' in the default Jupyter configuration directory.
     """
-
     def start(self):
         print('Application directory:   %s' % get_app_dir())
         print('User Settings directory: %s' % get_user_settings_dir())
@@ -400,7 +499,7 @@ class LabApp(NotebookApp):
         super(LabApp, self).init_server_extensions()
         msg = 'JupyterLab server extension not enabled, manually loading...'
         if not self.nbserver_extensions.get('jupyterlab', False):
-            self.log.warn(msg)
+            self.log.warning(msg)
             load_jupyter_server_extension(self)
 
 
